/**
 * \file core/tandem_run/tandemslurmprocess.h
 * \date 21/09/2023
 * \author Olivier Langella
 * \brief handles execution of a bunch of X!Tandem process through slurm
 * workload manager
 */

/*******************************************************************************
 * Copyright (c) 2023 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of i2MassChroQ.
 *
 *     i2MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     i2MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with i2MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include "tandemslurmprocess.h"
#include <QDebug>
#include <QSettings>
#include <QProcess>
#include <QThread>
#include <pappsomspp/pappsoexception.h>
#include "../../files/tandemparametersfile.h"
#include "../../utils/utils.h"

TandemSlurmProcess::TandemSlurmProcess(MainWindow *p_main_window,
                                       WorkMonitorInterface *p_monitor,
                                       const TandemRunBatch &tandem_run_batch)
  : TandemBatchProcess(p_main_window, p_monitor, tandem_run_batch)
{
  /*
  #!/bin/bash
#SBATCH --mail-user=olivier.langella@universite-paris-saclay.fr
#SBATCH --mail-type=end,fail
#SBATCH --partition=debug
#SBATCH --job-name="sbatch Example 01"
#SBATCH --cpus-per-task=10
# Your job commands
masschroq -t /tmp -c 10 balazadeh_dil.masschroqml

  */
  mp_slurmProcess = new SlurmProcess(p_main_window, p_monitor, "X!Tandem");

  QSettings settings;
  // m_condorRequestMemory =
  //  settings.value("tandem/condor_request_memory", "10000").toUInt();
}

TandemSlurmProcess::~TandemSlurmProcess()
{
  if(mp_slurmProcess != nullptr)
    {
      delete mp_slurmProcess;
    }
}

void
TandemSlurmProcess::prepareXmlDatabaseFile()
{
  /*
    QFile xml_database_file(QString("%1/database.xml").arg(mp_tmpDir->path()));

    if(xml_database_file.open(QIODevice::WriteOnly))
      {
        _xml_database_file =
          QFileInfo(xml_database_file.fileName()).absoluteFilePath();
        QXmlStreamWriter *p_out = new QXmlStreamWriter();
        p_out->setDevice(&xml_database_file);
        writeXmlDatabaseFile(p_out);
        xml_database_file.close();
        delete p_out;
      }
    else
      {
        throw pappso::PappsoException(
          QObject::tr("error : cannot open the XML database file : %1\n")
            .arg(xml_database_file.fileName()));
      }
      */
}

void
TandemSlurmProcess::run()
{
  qDebug();


  _p_monitor->finished(QObject::tr("%1 slurm X!Tandem job(s) finished")
                         .arg(_tandem_run_batch._mz_file_list.size()));
  qDebug() << "TandemCondorProcess::run end";
}
